
/*
 *
 * Copyright (C) 1999  Thomas Mirlacher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The author may be reached as dent@cosy.sbg.ac.at, or
 * Thomas Mirlacher, Jakob-Haringerstr. 2, A-5020 Salzburg,
 * Austria
 *
 *------------------------------------------------------------
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>

//for test purposes
#include <fcntl.h>

#include "buf.h"
#include "decoder.h"
#include <dvb/pes.h>


static struct {
	pthread_t loop;
	buf_t *buf;

	int end_loop;
	int fd;

	struct {
		u_char cont_count;
	} avp[_BUF_MAX];
} decoder;

static plugin_codec_t codec_sw = {
	DecoderInit,
	NULL,
	NULL,
	NULL,
	DecoderBufInit,
	DecoderBufExit,
	DecoderBufPut,	// read
	NULL,		// write
	DecoderPlay,
	DecoderPause,
	DecoderStop,
	DecoderSPU,
}; 

static int avp_encaps (buf_entry_t *buf_entry);


/**
 *
 */

plugin_codec_t *codec_init ()
{
	return &codec_sw;
}


/**
 *
 */

static void *codecLoop (void *foo)
{
	buf_entry_t buf_entry;

	while (!decoder.end_loop) {
		if (!DecoderBufGet (decoder.buf, BUF_VIDEO, &buf_entry)) {
// DENT: change 0xa0
			switch (buf_entry.buf_id) {
			case BUF_VIDEO:
			case BUF_AUDIO:
				avp_encaps (&buf_entry);
				break;
			default:
				break;
			}

			free (buf_entry.mem);
		} else {
//fprintf (stderr, "v");
//			pthread_cond_wait (&(decoder.buf->ctrl[BUF_VIDEO].cond_new_data), &(decoder.buf->ctrl[BUF_VIDEO].mutex_new_data));
		}
	}

	return NULL;
}


/**
 *
 */

int DecoderInit (buf_t *buf, char *output_path)
{
	decoder.buf = buf;
	
	fprintf (stderr, "DecoderInit\n");
	decoder.fd = open (output_path, O_WRONLY | O_CREAT);

	return 0;
}


/**
 *
 */

int DecoderPlay (void)
{
	fprintf (stderr, "DecoderPlay\n");
	decoder.end_loop = 0;
	pthread_create (&decoder.loop, NULL, codecLoop, NULL);

	return 0;
}


/**
 *
 */

int DecoderStop (void)
{
	fprintf (stderr, "DecoderStop\n");
	decoder.end_loop = 1;
	pthread_join (decoder.loop, NULL);

	return 0;
}


/**
 *
 */

int DecoderPause (void)
{
	fprintf (stderr, "DecoderPause\n");
	decoder.end_loop = 1;
	pthread_join (decoder.loop, NULL);

	return 0;
}


/**
 * n=-1 disable subpicture
 * n=0-31 enable subpicture n
 */

int DecoderSPU (int n)
{
	fprintf (stderr, "DecoderSPU\n");
	return 0;
}


/**
 * Set subpicture palette
 *
 */

int DecoderSPUCLUT (char *data, int size)
{
	fprintf (stderr, "DecoderSPUCLUT\n");
	return 0;
}

int DecoderSetAudioAttributes (int mode, int bits, int freq, int stream)
{
	return 0;
}

int DecoderSetVideoAttributes (int mode, int fps, int aspect_mode, int aspect_output, int tv_system)
{
	return 0;
}
#if 0

/* CSS authorization stuff */
int DecoderCSSDiscKey (char *data)
{
	return 0;
}

int DecoderCSSTitleKey (char *data, int cpm, int cp_sec, int cgms)
{
	return 0;
}

int DecoderCSSDriveAuthChallenge (char *data)
{
	return 0;
}

int DecoderCSSDriveAuthResponse (char *data)
{
	return 0;
}

int DecoderCSSHostAuth (char *data)
{
	return 0;
}

/* mode : 0=ac3, 1=mpeg, 2=lpcm */
/* bits : 0=16bit, 1=20bitm 2=24bit */
/* freq : 0=44.1kHz 1=48kHz 2=96kHz 3=22.05kHz 4=32kHz */
/* stream id : 0-7 */

/* mode : 0=VOB, 1=VCD, 2=MPG */
/* fps : 0=50Hz, 1=60Hz */
/* aspect_mode : 0=NORMAL, 1=LETTERBOX, 2=PANSCAN */
/* aspect_output: 0=4:3, 1=16:9 */
/* tv_system: 0=NTSC, 1=NTSC60, 2=PAL, 3=PALM, 4=PALM60, 5=PALN, 6=PALNc, 7=PAL60 */
int DecoderSetVideoAttributes (int mode, int fps, int aspect_mode, int aspect_output, int tv_system);

/* vol : 0-100 */
int DecoderAudioVolume (int vol);
#endif


static int avp_encaps (buf_entry_t *buf_entry)
{
	int len  = buf_entry->data_len; 
	char *inp = buf_entry->data;
	pes_hdr_t *pes;
	avp_hdr_t *avp;
	
	if ((buf_entry->data - buf_entry->mem) < AVP_HDR_LEN) {
		fprintf (stderr, "avp_encaps: need more headroom\n");
		return -1;
	}

	np -= PES_HDR_LEN + PES_FLAG_LEN;
        pes = (pes_hdr_t *) inp;

        pes->start_code_prefix = PES_START_CODE_PREFIX_RAW;
        pes->stream_id = 0xE0;
        pes->pes_packet_length_hi = (len>>8) & 0xFF;
        pes->pes_packet_length_lo = len & 0xFF;

        *(inp+PES_HDR_LEN) = 0; // set flags, and pes_header_data_length
        *(inp+PES_HDR_LEN+1) = 0;       // set flags, and pes_header_data_length
        *(inp+PES_HDR_LEN+2) = 0;       //
        len += PES_HDR_LEN + PES_FLAG_LEN;

	avp->prefix1 = 0x41;
	avp->prefix2 = 0x56;
	avp->stream_id = (buf_entry->buf_id == BUF_VIDEO) ? 0x01 : 0x02;
	avp->cont_count = priv.avp[buf_entry->buf_id].cont_count++;
	avp->prefix3 = 0x55;

	avp->length_hi = len >> 8;	// masking will be done by bitfield
	avp->length_lo = len;		// masking will be done by bitfield
}
