
/*
 * INPUT FROM DISK (using raw-i/o if possible)
 *
 * Copyright (C) 1999  Thomas Mirlacher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The author may be reached as dent@cosy.sbg.ac.at, or
 * Thomas Mirlacher, Jakob-Haringerstr. 2, A-5020 Salzburg,
 * Austria
 *
 *------------------------------------------------------------
 *
 */


//#define USE_RAWIO

#undef __REDIRECT
#define __USE_LARGEFILE64

#ifdef USE_RAWIO
#define NAME_RAWCTL "/dev/rawctl"
#define NAME_RAW "/dev/raw2"
#endif

#define DVD_VIDEO_LB_LEN 2048

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "plugin/input.h"
#include "../css/css.h"

#ifdef USE_RAWIO
#include <sys/ioctl.h>
#include <linux/raw.h>
#endif

static int udf_open (void);
static int udf_close (void);
static int udf_read (char *data, u_int len);
static int udf_seek (u_long pos);

static struct {
	__off64_t pos;
	int fd_raw;
	int fd;

	char key_disk[DVD_VIDEO_LB_LEN];
	char key_title[5];
} priv;

static plugin_input_t input = {
	PLUGIN_ID_INPUT,
	udf_open,		// open
	udf_close,		// close
	udf_read,		// read
	udf_seek,		// seek
	DVD_VIDEO_LB_LEN,	// blocksize
	&priv			// private data
};


/**
 *
 */

static int udf_open (void)
{
	if (CSSAuthTitle (priv.fd, priv.key_title, priv.pos) < 0)
		perror ("Authenticate title");

	if (CSSDecryptTitleKey (priv.key_title, priv.key_disk) < 0)
		perror ("Decrypting title");

	return 0;

}


/**
 *
 */

static int udf_close (void)
{
	return 0;
}


/**
 *
 */

static int udf_read (char *data, u_int len)
{
	int ret;
// do the CSS stuff in SW, just for now

	priv.pos++;

#ifdef USE_RAWIO
        if ((ret = read (priv.fd_raw, data, DVD_VIDEO_LB_LEN)) < 0)
		perror ("RAW");
#else
        if ((ret = read (priv.fd, data, DVD_VIDEO_LB_LEN)) < 0)
		perror ("UDF");
#endif
	CSSDescramble (data, priv.key_title);


	return ret;
}


/**
 *
 */

static int udf_seek (u_long pos)
{
	priv.pos = pos;

        if (lseek64 (priv.fd, (__off64_t) (pos)*DVD_VIDEO_LB_LEN, SEEK_SET) == -1) {
                fprintf (stderr, "%s/%d: error in lseek %x\n",
                        __FILE__, __LINE__, priv.pos);
                return -1;
        }

	return 0;
}


/**
 *
 */

static int open_rawdev (char *name)
{
#ifdef RAWIO
	struct raw_config_request raw_conf;
        struct stat info;
	int fd_rawctl;
	int fd_raw;

	if ((fd_rawctl = open (NAME_RAWCTL, O_RDWR)) < 0) {
		perror ("RAW: opening raw-io device");
		goto rawio_failed;
	}

        if (stat (name, &info) == -1)
                return -1;

// DENT: LOOP_START (looking for free rawdev)
	raw_conf.raw_minor = 2;

// DENT: look for a clean and future proof way to determine major/minor numbers
	raw_conf.block_major = (info.st_rdev >> 8) & 0xFF;
	raw_conf.block_minor = info.st_rdev & 0xFF;	

// DENT: LOOP_END (looking for free rawdev)
	if (ioctl (fd_rawctl, RAW_SETBIND, &raw_conf)) {
		perror ("RAW: ioctl raw io");
		close (fd_rawctl);
		goto rawio_failed;
	}

// check if raw IO setting are correct
	raw_conf.block_major = 0xFF;
        raw_conf.block_minor = 0xFF;

	if (ioctl (fd_rawctl, RAW_GETBIND, &raw_conf)) {
		perror ("RAW: ioctl raw io");
		close (fd_rawctl);
		goto rawio_failed;
	}
	fprintf (stderr, "RAW: raw_minor: %d\n\tblock_major: %ld\n\tblock_minor: %ld\n",raw_conf.raw_minor,raw_conf.block_major, raw_conf.block_minor);
// check end

	close (fd_rawctl);	// close ctrl interface

	if ((fd_raw = open (NAME_RAW, O_RDONLY)) < 0) {
                perror ("RAW: opening mapped raw-io device");
		goto rawio_failed;
        }

	fprintf (stderr, "RAW: everything went ok\n");
	return fd_raw;

rawio_failed:
	fprintf (stderr, "RAW: failed\n");
	return -1;
#endif
	return 0;
}


/**
 * Called upon init of input
 */

plugin_input_t *plugin_init (char *name)
{
	if ((priv.fd = open (name, O_RDONLY)) < 0) {
		perror ("opening udf disk");
		return NULL;
	}

#ifdef USE_RAWIO
	if ((priv.fd_raw = open_rawdev (name)) < 0)
		priv.fd_raw = priv.fd;
#endif

	if (CSSAuthDisk (priv.fd, priv.key_disk) < 0) {
		fprintf (stderr, "Error when authenticating\n");
//DENT: check here if disk is encrypted at all
	}

	return &input;
}


/**
 * Called when input is closed
 */

void plugin_exit (void)
{
	close (priv.fd_raw);
#ifdef USE_RAWIO
	if (priv.fd_raw != priv.fd)
#endif
		close (priv.fd);
}
