#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <linux/raw.h>

int raw_open (char *pathname);
int raw_close (int fd);

// c 162 0
#define NAME_RAWCTL "/dev/rawctl"
// c 162 1
#define NAME_RAW "/dev/raw1"

int main (int argc, char *argv[])
{
	char *buf;
	int fd_raw;
	int i;
	int ret;

	if (argc<2) {
		fprintf (stderr, "USAGE: %s device\n", argv[0]);
		exit (0);
	}

	buf = (char *) valloc (512);
	fprintf (stderr, "buf: %p\n", buf);

	if ((fd_raw = raw_open (argv[1])) < 0) {
		fprintf (stderr, "error opening RAW device\n");
		exit (-1);
	} else {
		fprintf (stderr, "successfully opened RAW device\n");
	}

	for (i=0; i<10; i++) {	// read 10 512b blocks
		ret = read (fd_raw, buf, 512);

		fprintf (stderr, "%d ", ret);
		if (ret < 0)
			perror ("read");
	}

	raw_close (fd_raw);

	free (buf);

	return 0;
}


/**
 *
 */

int raw_open (char *pathname)
{
	struct raw_config_request raw_conf;
	struct stat info;
	int fd_rawctl;
	int fd_raw;

	if ((fd_rawctl = open (NAME_RAWCTL, O_RDWR)) < 0) {
		perror ("RAW: opening raw-io device");
		goto rawio_failed;
	}

	if (stat (pathname, &info) == -1) {
		fprintf (stderr, "RAW: stat'ing %s\n", pathname);
		return -1;
	}

// DENT: LOOP_START (looking for free rawdev)
	raw_conf.raw_minor = 1;

// DENT: look for a clean and future proof way to determine major/minor numbers
	raw_conf.block_major = (info.st_rdev >> 8) & 0xFF;
	raw_conf.block_minor = info.st_rdev & 0xFF;

// DENT: LOOP_END (looking for free rawdev)
	if (ioctl (fd_rawctl, RAW_SETBIND, &raw_conf)) {
		perror ("RAW: ioctl raw io");
		close (fd_rawctl);
		goto rawio_failed;
	}

// check if raw IO setting are correct
#if 1
	raw_conf.block_major = 0xFF;
	raw_conf.block_minor = 0xFF;

	if (ioctl (fd_rawctl, RAW_GETBIND, &raw_conf)) {
		perror ("RAW: ioctl raw io");
		close (fd_rawctl);
		goto rawio_failed;
	}

        fprintf (stderr, "RAW: raw_minor: %d\n",
		raw_conf.raw_minor);
	fprintf (stderr, "\tblock_major: %ld\n\tblock_minor: %ld\n",
		raw_conf.block_major, raw_conf.block_minor);
#endif
// check end

	close (fd_rawctl);      // close ctrl interface

	if ((fd_raw = open (NAME_RAW, O_RDONLY)) < 0) {
		perror ("RAW: opening raw-io device");
		goto rawio_failed;
	}

        return fd_raw;

rawio_failed:
        return -1;
}


/**
 *
 */

int raw_close (int fd)
{
	return close (fd);
}
