#include <stdio.h>

#include "debug.h"
#include "Analyze.h"
#include "programs.h"
#include <dvb/eit.h>

#include "parse_eit.h"

unsigned char *parse_eit (unsigned char *buf, int len, int start)
{
	struct eit_struct *eit = (struct eit_struct *) buf;
	unsigned int section_length,
		event_id,
		service_id,
		this_length;
	unsigned char *ptr = buf;

	Log1 ("EIT_TABLE_ID: %x\n", eit->table_id);
	
	section_length = eit->section_length_hi<<8 | eit->section_length_lo;
	service_id = eit->service_id_hi<<8 | eit->service_id_lo;

	ptr += EIT_SIZE;

	if (!start)
		return NULL;

	if (section_length > len) 
		return NULL;
//		this_length= len;
	else
		this_length=section_length;

	while ((ptr-buf) < this_length) {
		unsigned int loop_length,
			desc_len;
		unsigned char *oldbuf = ptr;
		struct eit_loop_struct* eloop;

		desc_len = 0;		
		eloop = (struct eit_loop_struct *) ptr;

		event_id = eloop->event_id_hi<<8;
		event_id |= eloop->event_id_lo;

		Log1 ("EVENT_ID: %x\n", event_id);
		fprintf (stderr, "start time: %x:%x:%x\n",eloop->start_time_3,eloop->start_time_4,eloop->start_time_5);
		fprintf (stderr, "duration: %x:%x:%x\n",eloop->duration_1,eloop->duration_2,eloop->duration_3);
		loop_length = eloop->descriptors_loop_length_hi<<8;
		loop_length |= eloop->descriptors_loop_length_lo;

		Log1 ("service_id: %d", eit->service_id_hi<<8 | eit->service_id_lo);


		ptr += EIT_LOOP_SIZE;
		oldbuf = ptr;

// Descriptor Loop
		while ((ptr-oldbuf) < loop_length) {
			if (*ptr == EIT_SHORT_EVENT_DESCRIPTOR)
				ptr = (char *) parse_se (ptr, service_id);
			else {
				ptr += *(ptr+1)+2;
			}
		}
	}

	return ptr;
}

unsigned char *parse_se (unsigned char *buf, unsigned int service_id)
{
	unsigned char *ptr = buf;
	struct eit_short_event_descriptor_struct *eit_short;
	int i;
	struct program_struct *prog;
	eit_short = (struct eit_short_event_descriptor_struct *) ptr;

	prog = (struct program_struct *) add_program (service_id);

	prog->language[0] = eit_short->language_code_1;
	prog->language[1] = eit_short->language_code_2;
	prog->language[2] = eit_short->language_code_3;
	prog->language[3] = '\0';

	ptr += EIT_SHORT_EVENT_DESCRIPTOR_SIZE;

	fprintf (stderr, "\nEvent_id: %x", service_id);
	fprintf (stderr, "\nEvent_name: ");

	for (i=0; i<eit_short->event_name_length; i++) 
		fprintf (stderr, "%c", *ptr++);

	fprintf (stderr, "\nText: ");

	for (i=*ptr++; i>0; i--) 
		fprintf (stderr, "%c", *ptr++);

	fprintf (stderr, "\n");

	return ptr;
}
	
